package be.iminds.ilabt.jfed.lowlevel.authority;

import be.iminds.ilabt.jfed.lowlevel.testbed_info.TestbedInfoSource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/*
   Example https://www.emulab.net/protogeni/authorities/salist.txt:

   urn:publicid:IDN+uml.emulab.net+authority+sa https://boss.uml.emulab.net/protogeni/xmlrpc/sa
   urn:publicid:IDN+emulab.larc.usp.br+authority+sa https://www.emulab.larc.usp.br:12369/protogeni/xmlrpc/sa
   urn:publicid:IDN+emulab.net+authority+sa https://www.emulab.net:12369/protogeni/xmlrpc/sa
....
   urn:publicid:IDN+wall3.test.ibbt.be+authority+sa https://www.wall3.test.ibbt.be:12369/protogeni/xmlrpc/sa
 */

/**
 * Read jFed compatible XML representation of authorities
 * <p/>
 * URL:  https://flsmonitor.fed4fire.eu/testbeds.xml
 */
public class Fed4FireAuthorityList {
    public static class OfflineException extends Exception {
        public OfflineException() {
        }

        public OfflineException(String message) {
            super(message);
        }

        public OfflineException(String message, Throwable cause) {
            super(message, cause);
        }

        public OfflineException(Throwable cause) {
            super(cause);
        }

        public OfflineException(String message, Throwable cause, boolean enableSuppression, boolean writableStackTrace) {
            super(message, cause, enableSuppression, writableStackTrace);
        }
    }

    private static final Logger LOG = LoggerFactory.getLogger(Fed4FireAuthorityList.class);

    private static final String FED4FIRE_TESTBED_DIRECTORY_URL = "https://flsmonitor.fed4fire.eu/testbeds.xml";

    private Fed4FireAuthorityList() {
    }

    /* returns whether successful or not */
    public static boolean load(TestbedInfoSource testbedInfoSource) throws OfflineException {
        return load(testbedInfoSource, FED4FIRE_TESTBED_DIRECTORY_URL);
    }
    public static boolean load(TestbedInfoSource testbedInfoSource, String testbedXmlUrl) throws OfflineException {
        throw new RuntimeException("not yet reimplemented");
//        HttpParams params = new BasicHttpParams();
//        params.setParameter(CoreConnectionPNames.SO_TIMEOUT, 5000); //read timeout in ms (default: 0 = infinite)
//        params.setParameter(CoreConnectionPNames.CONNECTION_TIMEOUT, 10000); //connection timeout in ms (default: 0 = infinite)
//
//        DefaultHttpClient httpClient = new DefaultHttpClient(params);
//        X509Certificate cert = KeyUtil.pemToX509Certificate(FED4FIRE_TESTBED_DIRECTORY_SELFSIGNED_CERTIFICATE);
//        X509Certificate certOld = KeyUtil.pemToX509Certificate(FED4FIRE_TESTBED_DIRECTORY_SELFSIGNED_CERTIFICATE_OLD);
//        KeyStore trustStore = null;
//        try {
//            final URL serverUrl = new URL(testbedXmlUrl);
//
//            //start from empty
////            trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
////            trustStore.load(null, "any_pass".toCharArray());
//
//            //start from system trust store
//            trustStore = new JFedTrustStore().getTrustStore();
//
//            trustStore.setCertificateEntry("singleCert", cert);
//
//            trustStore.setCertificateEntry("singleCertOld", certOld);
//
//            String algo = SSLSocketFactory.TLS;
//            java.security.SecureRandom random = new SecureRandom();
//            org.apache.http.conn.ssl.X509HostnameVerifier hostnameVerifier =
//                    new org.apache.http.conn.ssl.AbstractVerifier() {
//                        private BrowserCompatHostnameVerifier base = new BrowserCompatHostnameVerifier();
//
//                        /**
//                         * @param host the hostname of the url to check.
//                         * @param cns the CN's of the certificate
//                         * @param subjectAlts the aliases fot the CN's that the certificate allows
//                         * @throws javax.net.ssl.SSLException
//                         */
//                        public final void verify(java.lang.String host, String[] cns, String[] subjectAlts) throws javax.net.ssl.SSLException {
//                            List<String> modifiedSubjectAlts = new ArrayList<String>();
//                            if (subjectAlts != null)
//                                for (String subjectAlt : subjectAlts)
//                                    modifiedSubjectAlts.add(subjectAlt);
//                            List<String> modifiedCns = new ArrayList<String>();
//                            if (cns != null)
//                                for (String cn : cns)
//                                    modifiedCns.add(cn);
//
//                            //force add of host itself, this makes any certificate we have stored match
//                            if (modifiedCns.contains(FED4FIRE_TESTBED_DIRECTORY_SELFSIGNED_CERTIFICATE_ALLOWED_ALIAS) ||
//                                    modifiedCns.contains(FED4FIRE_TESTBED_DIRECTORY_SELFSIGNED_CERTIFICATE_ALLOWED_ALIAS_OLD) ||
//                                    modifiedSubjectAlts.contains(FED4FIRE_TESTBED_DIRECTORY_SELFSIGNED_CERTIFICATE_ALLOWED_ALIAS) ||
//                                    modifiedSubjectAlts.contains(FED4FIRE_TESTBED_DIRECTORY_SELFSIGNED_CERTIFICATE_ALLOWED_ALIAS_OLD)
//                                    ) {
//                                modifiedSubjectAlts.add(serverUrl.getHost());
//                            }
//                            String[] newSubjectAlts = new String[modifiedSubjectAlts.size()];
//                            for (int i = 0; i < modifiedSubjectAlts.size(); i++)
//                                newSubjectAlts[i] = modifiedSubjectAlts.get(i);
//                            base.verify(host, cns, newSubjectAlts);
//                        }
//                    };
//            SSLSocketFactory socketFactory =
//                    new SSLSocketFactory(algo, null, null, trustStore, random, hostnameVerifier);
//            Scheme sch1 = new Scheme("https", 443, socketFactory); //port is the default port for the given protocol
//            httpClient.getConnectionManager().getSchemeRegistry().register(sch1);
//        } catch (Exception e) {
//            trustStore = null;
//            LOG.error("Could not create httpClient for Fed4FireAuthorityList: " + e, e);
//            throw new RuntimeException("Could not create httpClient for Fed4FireAuthorityList: " + e, e);
////            return false;
//        }
//
//
//        try {
//            URL startUrl = new URL(testbedXmlUrl);
//            HttpGet httpget = new HttpGet(startUrl.toExternalForm());
//            HttpResponse response = httpClient.execute(httpget);
//
//            if (response.getStatusLine() == null)
//                throw new RuntimeException("Error getting url \"" + testbedXmlUrl + "\" => " + response);
//            if (response.getStatusLine().getStatusCode() != 200)
//                throw new RuntimeException("Error getting url \"" + testbedXmlUrl + "\" (" + response.getStatusLine().getStatusCode() + ") => " + response.getStatusLine() + " => " + response);
//
//            HttpEntity entity = response.getEntity();
//            StoredTestbedInfo.LoadResult loadResult =  StoredTestbedInfo.load(entity.getContent(), testbedInfoSource);
//
//            for (String extraFile : loadResult.getExtraFiles()) {
//                URL url = new URL(startUrl, extraFile);
//                HttpGet httpget2 = new HttpGet(url.toURI());
//                LOG.trace("Loading extra file referenced from testbeds.xml: "+url.toExternalForm());
//                HttpResponse response2 = httpClient.execute(httpget2);
//
//                if (response2.getStatusLine() == null)
//                    throw new RuntimeException("Error getting url \"" + url + "\" => " + response2);
//                if (response2.getStatusLine().getStatusCode() != 200)
//                    throw new RuntimeException("Error getting url \"" + url + "\" (" + response2.getStatusLine().getStatusCode() + ") => " + response2.getStatusLine() + " => " + response2);
//
//                HttpEntity entity2 = response2.getEntity();
//                StoredTestbedInfo.LoadResult loadResult2 =  StoredTestbedInfo.load(entity2.getContent(), testbedInfoSource);
//                if (!loadResult2.isSuccess())
//                    loadResult = loadResult2;
//                if (!loadResult2.getExtraFiles().isEmpty())
//                    LOG.warn("Recursive extra files not supported (see "+url.toExternalForm()+")");
//            }
//
//            //TODO: this does not exist anymore:  testbedInfoSource.fireChange();
//            return loadResult.isSuccess();
//        } catch (javax.net.ssl.SSLPeerUnverifiedException ex) {
//            LOG.error("Fed4FireAuthorityList ERROR: SSLPeerUnverifiedException, The server certificate could not be verified.", ex);
//            return false;
//        } catch (ClientProtocolException e) {
//            LOG.error("Fed4FireAuthorityList ClientProtocolException: " + e.getMessage(), e);
//            return false;
//        } catch (HttpHostConnectException e) {
//            LOG.error("Fed4FireAuthorityList HttpHostConnectException: " + e.getMessage(), e);
//            throw new OfflineException(e);
//            //return false;
//        } catch (UnknownHostException e) {
//            LOG.error("Fed4FireAuthorityList UnknownHostException: " + e.getMessage(), e);
//            throw new OfflineException(e);
//            //return false;
//        } catch (ConnectException e) {
//            LOG.error("Fed4FireAuthorityList ConnectException: " + e.getMessage(), e);
//            throw new OfflineException(e);
//            //return false;
//        } catch (SocketException e) {
//            LOG.error("Fed4FireAuthorityList SocketException: " + e.getMessage(), e);
//            return false;
//        } catch (IOException e) {
//            LOG.error("Fed4FireAuthorityList IOException: " + e.getMessage(), e);
//            return false;
//        } catch (URISyntaxException e) {
//            LOG.error("Fed4FireAuthorityList URISyntaxException: " + e.getMessage(), e);
//            return false;
//        }
    }
}
